// src/index.ts
import jsTokens from "js-tokens";
var reservedWords = {
  keyword: [
    "break",
    "case",
    "catch",
    "continue",
    "debugger",
    "default",
    "do",
    "else",
    "finally",
    "for",
    "function",
    "if",
    "return",
    "switch",
    "throw",
    "try",
    "var",
    "const",
    "while",
    "with",
    "new",
    "this",
    "super",
    "class",
    "extends",
    "export",
    "import",
    "null",
    "true",
    "false",
    "in",
    "instanceof",
    "typeof",
    "void",
    "delete"
  ],
  strict: [
    "implements",
    "interface",
    "let",
    "package",
    "private",
    "protected",
    "public",
    "static",
    "yield"
  ]
}, keywords = new Set(reservedWords.keyword), reservedWordsStrictSet = new Set(reservedWords.strict), sometimesKeywords = /* @__PURE__ */ new Set(["as", "async", "from", "get", "of", "set"]);
function isReservedWord(word) {
  return word === "await" || word === "enum";
}
function isStrictReservedWord(word) {
  return isReservedWord(word) || reservedWordsStrictSet.has(word);
}
function isKeyword(word) {
  return keywords.has(word);
}
var BRACKET = /^[()[\]{}]$/, getTokenType = function(index, tokens) {
  let token = tokens[index];
  if (token.type === "IdentifierName") {
    if (isKeyword(token.value) || isStrictReservedWord(token.value) || sometimesKeywords.has(token.value))
      return "Keyword";
    if (token.value[0] && token.value[0] !== token.value[0].toLowerCase())
      return "IdentifierCapitalized";
    if (isPropertyFunction(index, tokens))
      return "IdentifierMethodName";
  }
  return token.type === "PrivateIdentifier" && isPropertyFunction(index, tokens) ? "PrivateIdentifierMethodName" : token.type === "Punctuator" && BRACKET.test(token.value) ? "Bracket" : token.type === "Invalid" && (token.value === "@" || token.value === "#") ? "Punctuator" : token.type;
};
function isPropertyFunction(index, tokens) {
  let token = tokens[++index];
  for (; token && (token.type === "WhiteSpace" || token.type === "LineTerminatorSequence" || token.type === "Punctuator" && (token.value === "(" || token.value === "?." || token.value === "!")); ) {
    if (token.type === "Punctuator" && token.value === "(")
      return !0;
    token = tokens[++index];
  }
  return !1;
}
var tokenize = function* (text, jsx = !1) {
  let tokens = Array.from(jsTokens(text, { jsx })), index = -1;
  for (let token of tokens)
    index++, yield {
      type: getTokenType(index, tokens),
      value: token.value
    };
};
function highlightTokens(defs, text, jsx) {
  let highlighted = "";
  for (let { type, value } of tokenize(text, jsx)) {
    let colorize = defs[type];
    colorize ? highlighted += colorize(value) : highlighted += value;
  }
  return highlighted;
}
function highlight(code, options = { jsx: !1, colors: {} }) {
  return code && highlightTokens(options.colors, code, options.jsx);
}

export {
  highlight
};
